@@ -1,5 +1,5 @@
 package Amazon::DynamoDB::NaHTTP;
-$Amazon::DynamoDB::NaHTTP::VERSION = '0.18';
+$Amazon::DynamoDB::NaHTTP::VERSION = '0.19';
 use strict;
 use warnings;
 
@@ -22,14 +22,27 @@ sub request {
 	my $self = shift;
 	my $req = shift;
 	my ($host, $port) = split /:/, ''.$req->uri->host_port;
+        my $resp;
 	$self->ua->do_request(
 		request => $req,
 		host    => $host,
 		port    => $port || 80,
-	)->transform(
+                on_response => sub {
+                    $resp = shift;
+                }
+	)-> transform(
 		done => sub {
-			shift->decoded_content
+                    if ($resp->is_success()) {
+                        return $resp->decoded_content;
+                    } else {
+                        my $status = join ' ', $resp->code, $resp->message;
+                        return Future->new->fail($status, $resp, $req)
+                    }
 		},
+                fail => sub {
+                    my $status = join ' ', $resp->code, $resp->message;
+                    return ($status, $resp, $req);
+                },
 	);
 }
 
@@ -84,7 +97,7 @@ Amazon::DynamoDB::NaHTTP
 
 =head1 VERSION
 
-version 0.18
+version 0.19
 
 =head1 DESCRIPTION
 
